@@ -12,7 +12,7 @@ use Shell::Carapace;
 use String::ShellQuote;
 use JSON::MaybeXS;
 
-our $VERSION = "0.06";
+our $VERSION = "0.07";
 
 =head1 NAME
 
@@ -22,10 +22,9 @@ Chef::Knife::Cmd - A small wrapper around the Chef 'knife' command line utility
 
     use Chef::Knife::Cmd;
 
+    # See Shell::Carapace for details about the callback attribute
     my $knife = Chef::Knife::Cmd->new(
-        verbose   => 0,           # if true, tee output to STDOUT; default is false
-        logfile   => 'knife.log',
-        print_cmd => 1,
+        callback => sub { ... }, # optional. useful for logging realtime output; 
     );
 
     # knife bootstrap
@@ -97,32 +96,25 @@ All commands return the output of the knife command.
 
 =item Logging
 
-All knife cmd output is logged to a logfile for later analysis.  This allows
-you to provide verbose logging to users while not cluttering their terminal
-with tons of output.
-
-=item Tee output to STDOUT
-
-Instead of waiting for an hour for a knife command to finish, you can view
-output from the knife command in real time as it happens.  This happens when
-the 'verbose' attribute is true.
+If you wish to log output, you should do so via the 'callback' attribute.  See
+Shell::Carapace for more details.
 
 =item Exceptions
 
-If a knife command fails, this module will throw an exception.
+If a knife command fails, an exception is thrown.
 
 =back
 
 =cut
 
-has verbose    => (is => 'rw', default => sub { 0 });
-has print_cmd  => (is => 'rw', default => sub { 0 });
 has noop       => (is => 'rw', default => sub { 0 });
 has shell      => (is => 'lazy');
-has logfile    => (is => 'ro');
 has format     => (is => 'rw');
 has _json_flag => (is => 'rw');
 
+has callback   => (is => 'rw');
+has output     => (is => 'rw');
+
 has client    => (is => 'lazy');
 has ec2       => (is => 'lazy');
 has node      => (is => 'lazy');
@@ -139,13 +131,23 @@ sub _build_data_bag  { Chef::Knife::Cmd::DataBag->new(knife => shift)   }
 
 sub _build_shell {
     my $self = shift;
-    my $shell = Shell::Carapace->new(
-        print_cmd => $self->print_cmd(),
-        verbose   => $self->verbose(),
-        noop      => $self->noop(),
-    );
-    $shell->logfile($self->logfile) if $self->logfile();
-    return $shell;
+    my $cb   = sub {
+        my ($type, $message) = @_;
+        if ($type ne 'error') {
+            if ($type eq 'command') {
+                $self->output('');
+            }
+            else {
+                my $output = '';
+                $output .= $self->output . "\n" if $self->output;
+                $output .= $message;
+                $self->output($output);
+            }
+        }
+        $self->callback->(@_) if $self->callback;
+    };
+
+    return Shell::Carapace->new(callback => $cb);
 }
 
 sub bootstrap {
@@ -178,7 +180,9 @@ sub handle_options {
 
 sub run {
     my ($self, @cmds) = @_;
-    my $out = $self->shell->local(@cmds);
+    return shell_quote @cmds if $self->noop;
+    $self->shell->local(@cmds);
+    my $out = $self->output;
     return JSON->new->utf8->decode($out) if $self->_json_flag;
     return $out;
 }
